@@ -26,7 +26,7 @@
 # POSSIBILITY OF SUCH DAMAGE.
 
 from u2flib_server import u2f
-from soft_u2f_v2 import SoftU2FDevice
+from .soft_u2f_v2 import SoftU2FDevice
 import unittest
 
 APP_ID = 'https://www.example.com'
@@ -75,5 +75,5 @@ class AttestationTest(unittest.TestCase):
         )
 
         assert u2f.verify_authenticate([device1, device2],
-                                    auth_request_data,
-                                    response)
+                                       auth_request_data,
+                                       response)
