@@ -25,6 +25,11 @@
 # ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 # POSSIBILITY OF SUCH DAMAGE.
 
+from cryptography.x509 import (
+    ExtensionNotFound, ObjectIdentifier
+)
+
+
 __all__ = [
     'DeviceMatcher',
     'FingerprintMatcher',
@@ -44,20 +49,18 @@ class FingerprintMatcher(DeviceMatcher):
     selector_type = 'fingerprint'
 
     def matches(self, certificate, parameters=[]):
-        fingerprints = map(lambda s: s.lower(), parameters)
+        fingerprints = [s.lower() for s in parameters]
         return certificate.get_fingerprint('sha1').lower() in fingerprints
 
 
-# This is needed since older versions of M2Crypto don't have a way of getting
-# extensions by their OID.
 def get_ext_by_oid(cert, oid):
-    from pyasn1.codec.der import decoder
-    from pyasn1_modules import rfc2459
-    cert, _ = decoder.decode(cert.as_der(), asn1Spec=rfc2459.Certificate())
-    for ext in cert['tbsCertificate']['extensions']:
-        if ext['extnID'].prettyPrint() == oid:
-            return decoder.decode(ext['extnValue'])[0].asOctets()
-    return None
+    oid = ObjectIdentifier(oid)
+    try:
+        extension = cert.extensions.get_extension_for_oid(oid)
+    except ExtensionNotFound:
+        return None
+
+    return extension.value.value
 
 
 class ExtensionMatcher(DeviceMatcher):
