@@ -27,12 +27,18 @@
 
 __all__ = ['MetadataResolver', 'create_resolver']
 
-from M2Crypto import X509
 from u2flib_server.jsapi import MetadataObject
 from u2flib_server.attestation.data import YUBICO
+from u2flib_server.utils import verify_cert_signature
+from u2flib_server.yubicommon.compat import string_types, text_type
 import os
 import json
 
+from cryptography import x509
+from cryptography.exceptions import InvalidSignature
+from cryptography.hazmat.backends import default_backend
+from cryptography.x509.oid import NameOID
+
 
 class MetadataResolver(object):
 
@@ -61,17 +67,33 @@ class MetadataResolver(object):
 
     def _index(self, metadata):
         for cert_pem in metadata.trustedCertificates:
-            cert_der = ''.join(cert_pem.splitlines()[1:-1]).decode('base64')
-            cert = X509.load_cert_der_string(cert_der)
-            subject = cert.get_subject().as_text()
+            if isinstance(cert_pem, text_type):
+                cert_pem = cert_pem.encode('ascii')
+            cert = x509.load_pem_x509_certificate(cert_pem, default_backend())
+            subject = cert.subject.get_attributes_for_oid(NameOID.COMMON_NAME)[0].value
             if subject not in self._certs:
                 self._certs[subject] = []
             self._certs[subject].append(cert)
             self._metadata[cert] = metadata
 
+    def _verify_cert(self, cert, key):
+        """Returns True if cert contains a correct signature made using the
+        provided key
+
+        NB: This *only* checks the signature. No other checks are performed.
+        E.g. the trust chain, expiry are all ignored.
+        """
+        try:
+            verify_cert_signature(cert, key)
+            return True
+        except InvalidSignature:
+            return False
+
     def resolve(self, cert):
-        for issuer in self._certs.get(cert.get_issuer().as_text(), []):
-            if cert.verify(issuer.get_pubkey()) == 1:
+        issuer = cert.issuer.get_attributes_for_oid(NameOID.COMMON_NAME)[0].value
+
+        for issuer in self._certs.get(issuer, []):
+            if self._verify_cert(cert, issuer.public_key()):
                 return self._metadata[issuer]
         return None
 
@@ -82,9 +104,9 @@ def _load_from_file(fname):
 
 
 def _load_from_dir(dname):
-    return map(_load_from_file,
-               [os.path.join(dname, d) for d in os.listdir(dname)
-                if d.endswith('.json')])
+    json_fnames = [os.path.join(dname, d) for d in os.listdir(dname)
+                   if d.endswith('.json')]
+    return [_load_from_file(fname) for fname in json_fnames]
 
 
 def _add_data(resolver, data):
@@ -92,7 +114,7 @@ def _add_data(resolver, data):
         for d in data:
             _add_data(resolver, d)
         return
-    elif isinstance(data, basestring):
+    elif isinstance(data, string_types):
         if os.path.isdir(data):
             data = _load_from_dir(data)
         elif os.path.isfile(data):
